@@ -37,13 +37,16 @@
    
    Other macros:
 
-   __int64                 define to be long long. Using a typedef can
-                           tweak bugs in the C++ parser.
+   __int64                 define to be long long. Using a typedef doesn't
+                           work for "unsigned __int64"
   
    All headers should include this first, and then use __DECLSPEC_SUPPORTED
    to choose between the old ``__imp__name'' style or __MINGW_IMPORT
    style declarations.  */
 
+/* Try to avoid problems with outdated checks for GCC __attribute__ support.  */ 
+#undef __attribute__
+
 #ifndef __GNUC__
 # ifndef __MINGW_IMPORT
 #  define __MINGW_IMPORT  __declspec(dllimport)
@@ -52,16 +55,17 @@
 #  define _CRTIMP  __declspec(dllimport)
 # endif
 # define __DECLSPEC_SUPPORTED
+# define __attribute__(x) /* nothing */
 #else /* __GNUC__ */
 # ifdef __declspec
 #  ifndef __MINGW_IMPORT
    /* Note the extern. This is needed to work around GCC's
       limitations in handling dllimport attribute.  */
-#   define __MINGW_IMPORT  extern __attribute__((dllimport))
+#   define __MINGW_IMPORT  extern __attribute__ ((dllimport))
 #  endif
 #  ifndef _CRTIMP
 #   ifdef __USE_CRTIMP
-#    define _CRTIMP  __attribute__((dllimport))
+#    define _CRTIMP  __attribute__ ((dllimport))
 #   else  
 #    define _CRTIMP
 #   endif
@@ -75,10 +79,10 @@
 #  endif
 # endif /* __declspec */
 # ifndef __cdecl
-#  define __cdecl __attribute__((cdecl))
+#  define __cdecl __attribute__ ((__cdecl__))
 # endif
 # ifndef __stdcall
-#  define __stdcall __attribute__((stdcall))
+#  define __stdcall __attribute__ ((__stdcall__))
 # endif
 # ifndef __int64
 #  define __int64 long long
@@ -87,7 +91,7 @@
 #  define __int32 long
 # endif
 # ifndef __int16
-#  define __int16 int
+#  define __int16 short
 # endif
 # ifndef __int8
 #  define __int8 char
@@ -106,8 +110,56 @@
 #define __CRT_INLINE extern __inline__
 #endif
 
-#define __MINGW32_VERSION 3.2
+#if defined (__GNUC__) && defined (__GNUC_MINOR__)
+#define __MINGW_GNUC_PREREQ(major, minor) \
+  (__GNUC__ > (major) \
+   || (__GNUC__ == (major) && __GNUC_MINOR__ >= (minor)))
+#else
+#define __MINGW_GNUC_PREREQ(major, minor)  0
+#endif
+
+#ifdef __cplusplus
+# define __UNUSED_PARAM(x) 
+#else 
+# ifdef __GNUC__
+#  define __UNUSED_PARAM(x) x __attribute__ ((__unused__))
+# else
+#  define __UNUSED_PARAM(x) x
+# endif
+#endif
+
+#ifdef __GNUC__
+#define __MINGW_ATTRIB_NORETURN __attribute__ ((__noreturn__))
+#define __MINGW_ATTRIB_CONST __attribute__ ((__const__))
+#else
+#define __MINGW_ATTRIB_NORETURN
+#define __MINGW_ATTRIB_CONST
+#endif
+
+#if __MINGW_GNUC_PREREQ (3, 0)
+#define __MINGW_ATTRIB_MALLOC __attribute__ ((__malloc__))
+#define __MINGW_ATTRIB_PURE __attribute__ ((__pure__))
+#else
+#define __MINGW_ATTRIB_MALLOC
+#define __MINGW_ATTRIB_PURE
+#endif
+
+/* Attribute `nonnull' was valid as of gcc 3.3.  We don't use GCC's
+   variadiac macro facility, because variadic macros cause syntax
+   errors with  --traditional-cpp.  */
+#if  __MINGW_GNUC_PREREQ (3, 3)
+#define __MINGW_ATTRIB_NONNULL(arg) __attribute__ ((__nonnull__ (arg)))
+#else
+#define __MINGW_ATTRIB_NONNULL(arg)
+#endif /* GNUC >= 3.3 */
+
+#ifndef __MSVCRT_VERSION__
+/*  High byte is the major version, low byte is the minor. */
+# define __MSVCRT_VERSION__ 0x0600
+#endif
+
+#define __MINGW32_VERSION 3.9
 #define __MINGW32_MAJOR_VERSION 3
-#define __MINGW32_MINOR_VERSION 2
+#define __MINGW32_MINOR_VERSION 9
 
 #endif /* __MINGW_H */
