@@ -1,27 +1,11 @@
 /*
  * stdlib.h
+ * This file has no copyright assigned and is placed in the Public Domain.
+ * This file is a part of the mingw-runtime package.
+ * No warranty is given; refer to the file DISCLAIMER within the package.
  *
  * Definitions for common types, variables, and functions.
  *
- * This file is part of the Mingw32 package.
- *
- * Contributors:
- *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
- *
- *  THIS SOFTWARE IS NOT COPYRIGHTED
- *
- *  This source code is offered for use in the public domain. You may
- *  use, modify or distribute it freely.
- *
- *  This code is distributed in the hope that it will be useful but
- *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
- *  DISCLAIMED. This includes but is not limited to warranties of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * $Revision: 1.19 $
- * $Author: earnie $
- * $Date: 2003/10/10 15:02:04 $
- *
  */
 
 #ifndef _STDLIB_H_
@@ -317,12 +301,6 @@ __MINGW_IMPORT  int _fmode_dll;
 
 #endif /* Not __STRICT_ANSI__ */
 
-#ifdef	__GNUC__
-#define	_ATTRIB_NORETURN	__attribute__ ((noreturn))
-#else	/* Not __GNUC__ */
-#define	_ATTRIB_NORETURN
-#endif	/* __GNUC__ */
-
 _CRTIMP double __cdecl	atof	(const char*);
 _CRTIMP int __cdecl	atoi	(const char*);
 _CRTIMP long __cdecl 	atol	(const char*);
@@ -332,8 +310,9 @@ _CRTIMP long __cdecl _wtol (const wchar_t *);
 #endif
 _CRTIMP double __cdecl	strtod	(const char*, char**);
 #if !defined __NO_ISOCEXT  /* extern stub in static libmingwex.a */
-__CRT_INLINE float __cdecl strtof (const char *nptr, char **endptr)
-  { return (strtod (nptr, endptr));}
+float __cdecl strtof (const char *, char **);
+__CRT_INLINE float __cdecl strtof (const char *__nptr, char **__endptr)
+  { return (strtod (__nptr, __endptr));}
 long double __cdecl strtold (const char * __restrict__, char ** __restrict__);
 #endif /* __NO_ISOCEXT */
 
@@ -344,8 +323,9 @@ _CRTIMP unsigned long __cdecl	strtoul	(const char*, char**, int);
 /*  also declared in wchar.h */
 _CRTIMP double __cdecl	wcstod	(const wchar_t*, wchar_t**);
 #if !defined __NO_ISOCEXT /* extern stub in static libmingwex.a */
-__CRT_INLINE float __cdecl wcstof( const wchar_t *nptr, wchar_t **endptr)
-{  return (wcstod(nptr, endptr)); }
+float __cdecl wcstof( const wchar_t *, wchar_t **);
+__CRT_INLINE float __cdecl wcstof( const wchar_t *__nptr, wchar_t **__endptr)
+{  return (wcstod(__nptr, __endptr)); }
 long double __cdecl wcstold (const wchar_t * __restrict__, wchar_t ** __restrict__);
 #endif /* __NO_ISOCEXT */
 
@@ -364,13 +344,13 @@ _CRTIMP int __cdecl	mbtowc		(wchar_t*, const char*, size_t);
 _CRTIMP int __cdecl	rand	(void);
 _CRTIMP void __cdecl	srand	(unsigned int);
 
-_CRTIMP void* __cdecl	calloc	(size_t, size_t);
-_CRTIMP void* __cdecl	malloc	(size_t);
+_CRTIMP void* __cdecl	calloc	(size_t, size_t) __MINGW_ATTRIB_MALLOC;
+_CRTIMP void* __cdecl	malloc	(size_t) __MINGW_ATTRIB_MALLOC;
 _CRTIMP void* __cdecl	realloc	(void*, size_t);
 _CRTIMP void __cdecl	free	(void*);
 
-_CRTIMP void __cdecl	abort	(void) _ATTRIB_NORETURN;
-_CRTIMP void __cdecl	exit	(int) _ATTRIB_NORETURN;
+_CRTIMP void __cdecl	abort	(void) __MINGW_ATTRIB_NORETURN;
+_CRTIMP void __cdecl	exit	(int) __MINGW_ATTRIB_NORETURN;
 
 /* Note: This is in startup code, not imported directly from dll */
 int __cdecl	atexit	(void (*)(void));
@@ -384,8 +364,8 @@ _CRTIMP void* __cdecl	bsearch	(const void*, const void*, size_t, size_t,
 _CRTIMP void __cdecl	qsort	(void*, size_t, size_t,
 				 int (*)(const void*, const void*));
 
-_CRTIMP int __cdecl	abs	(int);
-_CRTIMP long __cdecl	labs	(long);
+_CRTIMP int __cdecl	abs	(int) __MINGW_ATTRIB_CONST;
+_CRTIMP long __cdecl	labs	(long) __MINGW_ATTRIB_CONST;
 
 /*
  * div_t and ldiv_t are structures used to return the results of div and
@@ -398,8 +378,8 @@ _CRTIMP long __cdecl	labs	(long);
 typedef struct { int quot, rem; } div_t;
 typedef struct { long quot, rem; } ldiv_t;
 
-_CRTIMP div_t __cdecl	div	(int, int);
-_CRTIMP ldiv_t __cdecl	ldiv	(long, long);
+_CRTIMP div_t __cdecl	div	(int, int) __MINGW_ATTRIB_CONST;
+_CRTIMP ldiv_t __cdecl	ldiv	(long, long) __MINGW_ATTRIB_CONST;
 
 #if !defined (__STRICT_ANSI__)
 
@@ -411,7 +391,7 @@ _CRTIMP void __cdecl	_beep (unsigned int, unsigned int);
 _CRTIMP void __cdecl	_seterrormode (int);
 _CRTIMP void __cdecl	_sleep (unsigned long);
 
-_CRTIMP void __cdecl	_exit	(int) _ATTRIB_NORETURN;
+_CRTIMP void __cdecl	_exit	(int) __MINGW_ATTRIB_NORETURN;
 
 /* _onexit is MS extension. Use atexit for portability.  */
 /* Note: This is in startup code, not imported directly from dll */
@@ -452,10 +432,17 @@ _CRTIMP void __cdecl    _wmakepath(wchar_t*, const wchar_t*, const wchar_t*, con
 _CRTIMP void __cdecl	_wsplitpath (const wchar_t*, wchar_t*, wchar_t*, wchar_t*, wchar_t*);
 _CRTIMP wchar_t* __cdecl    _wfullpath (wchar_t*, const wchar_t*, size_t);
 
-_CRTIMP unsigned int __cdecl _rotl(unsigned int, int);
-_CRTIMP unsigned int __cdecl _rotr(unsigned int, int);
-_CRTIMP unsigned long __cdecl _lrotl(unsigned long, int);
-_CRTIMP unsigned long __cdecl _lrotr(unsigned long, int);
+_CRTIMP unsigned int __cdecl _rotl(unsigned int, int) __MINGW_ATTRIB_CONST;
+_CRTIMP unsigned int __cdecl _rotr(unsigned int, int) __MINGW_ATTRIB_CONST;
+_CRTIMP unsigned long __cdecl _lrotl(unsigned long, int) __MINGW_ATTRIB_CONST;
+_CRTIMP unsigned long __cdecl _lrotr(unsigned long, int) __MINGW_ATTRIB_CONST;
+
+_CRTIMP int __cdecl _set_error_mode (int);
+#define _OUT_TO_DEFAULT	0
+#define _OUT_TO_STDERR	1
+#define _OUT_TO_MSGBOX	2
+#define _REPORT_ERRMODE	3
+
 #endif
 
 #ifndef	_NO_OLDNAMES
@@ -480,16 +467,17 @@ _CRTIMP char* __cdecl	gcvt (double, int, char*);
 #if !defined __NO_ISOCEXT /* externs in static libmingwex.a */
 
 /* C99 name for _exit */
-void __cdecl _Exit(int) _ATTRIB_NORETURN;
+void __cdecl _Exit(int) __MINGW_ATTRIB_NORETURN;
 #ifndef __STRICT_ANSI__   /* inline using non-ansi functions */
-__CRT_INLINE void __cdecl _Exit(int status)
-	{  _exit(status); }
+__CRT_INLINE void __cdecl _Exit(int __status)
+	{  _exit (__status); }
 #endif 
 
 typedef struct { long long quot, rem; } lldiv_t;
 
-lldiv_t	__cdecl lldiv (long long, long long);
+lldiv_t	__cdecl lldiv (long long, long long) __MINGW_ATTRIB_CONST;
 
+long long __cdecl llabs(long long);
 __CRT_INLINE long long __cdecl llabs(long long _j)
   {return (_j >= 0 ? _j : -_j);}
 
@@ -525,10 +513,6 @@ __CRT_INLINE wchar_t*  __cdecl ulltow (unsigned long long _n, wchar_t * _w, int
 
 #endif /* !__NO_ISOCEXT */
 
-/*
- * Undefine the no return attribute used in some function definitions
- */
-#undef	_ATTRIB_NORETURN
 
 #ifdef __cplusplus
 }
