@@ -173,7 +173,7 @@
 #define ERROR_NESTING_NOT_ALLOWED 215L
 #define ERROR_EXE_MACHINE_TYPE_MISMATCH 216L
 #define ERROR_EXE_CANNOT_MODIFY_SIGNED_BINARY 217L
-#define ERRO_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY 218L
+#define ERROR_EXE_CANNOT_MODIFY_STRONG_SIGNED_BINARY 218L
 #define ERROR_BAD_PIPE 230L
 #define ERROR_PIPE_BUSY 231L
 #define ERROR_NO_DATA 232L
@@ -2079,6 +2079,7 @@
 #define MEM_E_INVALID_ROOT ((HRESULT)0x80080009L)
 #define MEM_E_INVALID_LINK ((HRESULT)0x80080010L)
 #define MEM_E_INVALID_SIZE ((HRESULT)0x80080011L)
+#define CO_S_NOTALLINTERFACES ((HRESULT)0x00080012L)
 #define DISP_E_UNKNOWNINTERFACE ((HRESULT)0x80020001L)
 #define DISP_E_MEMBERNOTFOUND ((HRESULT)0x80020003L)
 #define DISP_E_PARAMNOTFOUND ((HRESULT)0x80020004L)
@@ -2095,6 +2096,7 @@
 #define DISP_E_PARAMNOTOPTIONAL ((HRESULT)0x8002000FL)
 #define DISP_E_BADCALLEE ((HRESULT)0x80020010L)
 #define DISP_E_NOTACOLLECTION ((HRESULT)0x80020011L)
+#define DISP_E_DIVBYZERO ((HRESULT)0x80020012L)
 #define TYPE_E_BUFFERTOOSMALL ((HRESULT)0x80028016L)
 #define TYPE_E_INVDATAREAD ((HRESULT)0x80028018L)
 #define TYPE_E_UNSUPFORMAT ((HRESULT)0x80028019L)
