@@ -1,5 +1,9 @@
 #ifndef _WININET_H
 #define _WININET_H
+#if __GNUC__ >=3
+#pragma GCC system_header
+#endif
+
 #include <windows.h>
 #ifdef __cplusplus
 extern "C" {
@@ -140,6 +144,9 @@ INTERNET_FLAG_NO_COOKIES|INTERNET_FLAG_NO_AUTH|SECURITY_INTERNET_MASK|INTERNET_F
 #define SECURITY_FLAG_PCT 8
 #define SECURITY_FLAG_PCT4 16
 #define SECURITY_FLAG_IETFSSL4 0x20
+#define SECURITY_FLAG_IGNORE_REVOCATION 0x00000080
+#define SECURITY_FLAG_IGNORE_UNKNOWN_CA 0x00000100
+#define SECURITY_FLAG_IGNORE_WRONG_USAGE 0x00000200
 #define SECURITY_FLAG_40BIT 0x10000000
 #define SECURITY_FLAG_128BIT 0x20000000
 #define SECURITY_FLAG_56BIT 0x40000000
@@ -449,6 +456,10 @@ GOPHER_TYPE_UNIX_UUENCODED|GOPHER_TYPE_BINARY|GOPHER_TYPE_GIF|GOPHER_TYPE_IMAGE|
 #define CACHE_ENTRY_ACCTIME_FC 0x100
 #define CACHE_ENTRY_SYNCTIME_FC 0x200
 #define CACHE_ENTRY_HEADERINFO_FC 0x400
+#define IRF_ASYNC WININET_API_FLAG_ASYNC
+#define IRF_SYNC WININET_API_FLAG_SYNC
+#define IRF_USE_CONTEXT WININET_API_FLAG_USE_CONTEXT
+#define IRF_NO_WAIT 8
 #define HSR_ASYNC	WININET_API_FLAG_ASYNC
 #define HSR_SYNC	WININET_API_FLAG_SYNC
 #define HSR_USE_CONTEXT WININET_API_FLAG_USE_CONTEXT
@@ -462,14 +473,25 @@ GOPHER_TYPE_UNIX_UUENCODED|GOPHER_TYPE_BINARY|GOPHER_TYPE_GIF|GOPHER_TYPE_IMAGE|
 #define INTERNET_AUTODIAL_FORCE_ONLINE	1
 #define INTERNET_AUTODIAL_FORCE_UNATTENDED	2
 #define INTERNET_AUTODIAL_FAILIFSECURITYCHECK	4
-#define INTERNET_CONNECTION_MODEM	1
-#define INTERNET_CONNECTION_LAN	2
-#define INTERNET_CONNECTION_PROXY	4
-#define INTERNET_CONNECTION_MODEM_BUSY	8
+#define INTERNET_CONNECTION_MODEM	0x01
+#define INTERNET_CONNECTION_LAN		0x02
+#define INTERNET_CONNECTION_PROXY	0x04
+#define INTERNET_CONNECTION_MODEM_BUSY	0x08
+#define INTERNET_RAS_INSTALLED		0x10
+#define INTERNET_CONNECTION_OFFLINE	0x20
+#define INTERNET_CONNECTION_CONFIGURED	0x40
+#define CACHEGROUP_SEARCH_ALL	0
+#define CACHEGROUP_SEARCH_BYURL	1
+#define INTERNET_CACHE_GROUP_ADD	0
+#define INTERNET_CACHE_GROUP_REMOVE	1
+#define WININET_API_FLAG_ASYNC 0x00000001	/* Force asynchronous operations.  */
+#define WININET_API_FLAG_SYNC 0x00000004	/* Force synchronous operations.  */
+#define WININET_API_FLAG_USE_CONTEXT 0x00000008	/* Force the API to use the context value, even if it is set to zero.  */
 
 #ifndef RC_INVOKED
 typedef PVOID HINTERNET;
 typedef HINTERNET *LPHINTERNET;
+typedef LONGLONG GROUPID;
 typedef WORD INTERNET_PORT,*LPINTERNET_PORT;
 typedef enum {
 	INTERNET_SCHEME_PARTIAL = -2, INTERNET_SCHEME_UNKNOWN = -1,
@@ -676,6 +698,27 @@ typedef struct _INTERNET_BUFFERSW {
 	DWORD dwOffsetHigh;
 } INTERNET_BUFFERSW,*LPINTERNET_BUFFERSW;
 
+#define GROUP_OWNER_STORAGE_SIZE 4
+#define GROUPNAME_MAX_LENGTH 120
+typedef struct _INTERNET_CACHE_GROUP_INFOA {
+	DWORD dwGroupSize;
+	DWORD dwGroupFlags;
+	DWORD dwGroupType;
+	DWORD dwDiskUsage;
+	DWORD dwDiskQuota;
+	DWORD dwOwnerStorage[GROUP_OWNER_STORAGE_SIZE];
+	CHAR  szGroupName[GROUPNAME_MAX_LENGTH];
+} INTERNET_CACHE_GROUP_INFOA, * LPINTERNET_CACHE_GROUP_INFOA;
+typedef struct _INTERNET_CACHE_GROUP_INFOW {
+	DWORD dwGroupSize;
+	DWORD dwGroupFlags;
+	DWORD dwGroupType;
+	DWORD dwDiskUsage;
+	DWORD dwDiskQuota;
+	DWORD dwOwnerStorage[GROUP_OWNER_STORAGE_SIZE];
+	WCHAR szGroupName[GROUPNAME_MAX_LENGTH];
+} INTERNET_CACHE_GROUP_INFOW, * LPINTERNET_CACHE_GROUP_INFOW;
+
 #ifdef UNICODE
 typedef URL_COMPONENTSW URL_COMPONENTS;
 typedef LPURL_COMPONENTSW LPURL_COMPONENTS;
@@ -684,6 +727,8 @@ typedef LPGOPHER_FIND_DATAW LPGOPHER_FIND_DATA;
 typedef INTERNET_CACHE_ENTRY_INFOW INTERNET_CACHE_ENTRY_INFO;
 typedef LPINTERNET_CACHE_ENTRY_INFOW LPINTERNET_CACHE_ENTRY_INFO;
 typedef INTERNET_BUFFERSW INTERNET_BUFFERS,*LPINTERNET_BUFFERS;
+typedef INTERNET_CACHE_GROUP_INFOW INTERNET_CACHE_GROUP_INFO;
+typedef LPINTERNET_CACHE_GROUP_INFOW LPINTERNET_CACHE_GROUP_INFO;
 #else
 typedef URL_COMPONENTSA URL_COMPONENTS;
 typedef LPURL_COMPONENTSA LPURL_COMPONENTS;
@@ -692,6 +737,8 @@ typedef LPGOPHER_FIND_DATAA LPGOPHER_FIND_DATA;
 typedef INTERNET_CACHE_ENTRY_INFOA INTERNET_CACHE_ENTRY_INFO;
 typedef LPINTERNET_CACHE_ENTRY_INFOA LPINTERNET_CACHE_ENTRY_INFO;
 typedef INTERNET_BUFFERSA INTERNET_BUFFERS,*LPINTERNET_BUFFERS;
+typedef INTERNET_CACHE_GROUP_INFOA INTERNET_CACHE_GROUP_INFO;
+typedef LPINTERNET_CACHE_GROUP_INFOA LPINTERNET_CACHE_GROUP_INFO;
 #endif
 BOOL WINAPI InternetTimeFromSystemTime(SYSTEMTIME*,DWORD,LPSTR,DWORD);
 BOOL WINAPI InternetTimeToSystemTime(LPCSTR,SYSTEMTIME*,DWORD);
@@ -702,6 +749,8 @@ BOOL WINAPI InternetCreateUrlA(LPURL_COMPONENTSA,DWORD,LPSTR,PDWORD);
 BOOL WINAPI InternetCreateUrlW(LPURL_COMPONENTSW,DWORD,LPWSTR,PDWORD);
 BOOL WINAPI InternetCanonicalizeUrlA(LPCSTR,LPSTR,PDWORD,DWORD);
 BOOL WINAPI InternetCanonicalizeUrlW(LPCWSTR,LPWSTR,PDWORD,DWORD);
+BOOL WINAPI InternetCheckConnectionA(LPCSTR,DWORD,DWORD);
+BOOL WINAPI InternetCheckConnectionW(LPCWSTR,DWORD,DWORD);
 BOOL WINAPI InternetCombineUrlA(LPCSTR,LPCSTR,LPSTR,PDWORD,DWORD);
 BOOL WINAPI InternetCombineUrlW(LPCWSTR,LPCWSTR,LPWSTR,PDWORD,DWORD);
 HINTERNET WINAPI InternetOpenA(LPCSTR,DWORD,LPCSTR,LPCSTR,DWORD);
@@ -726,6 +775,7 @@ BOOL WINAPI InternetSetOptionExW(HINTERNET,DWORD,PVOID,DWORD,DWORD);
 BOOL WINAPI InternetGetLastResponseInfoA(PDWORD,LPSTR,PDWORD);
 BOOL WINAPI InternetGetLastResponseInfoW(PDWORD,LPWSTR,PDWORD);
 INTERNET_STATUS_CALLBACK WINAPI InternetSetStatusCallback(HINTERNET,INTERNET_STATUS_CALLBACK);
+DWORD WINAPI FtpGetFileSize(HINTERNET,LPDWORD);
 HINTERNET WINAPI FtpFindFirstFileA(HINTERNET,LPCSTR,LPWIN32_FIND_DATA,DWORD,DWORD);
 HINTERNET WINAPI FtpFindFirstFileW(HINTERNET,LPCWSTR,LPWIN32_FIND_DATA,DWORD,DWORD);
 BOOL WINAPI FtpGetFileA(HINTERNET,LPCSTR,LPCSTR,BOOL,DWORD,DWORD,DWORD);
@@ -746,8 +796,8 @@ BOOL WINAPI FtpSetCurrentDirectoryA(HINTERNET,LPCSTR);
 BOOL WINAPI FtpSetCurrentDirectoryW(HINTERNET,LPCWSTR);
 BOOL WINAPI FtpGetCurrentDirectoryA(HINTERNET,LPSTR,PDWORD);
 BOOL WINAPI FtpGetCurrentDirectoryW(HINTERNET,LPWSTR,PDWORD);
-BOOL WINAPI FtpCommandA(HINTERNET,BOOL,DWORD,LPCSTR,DWORD);
-BOOL WINAPI FtpCommandW( HINTERNET,BOOL,DWORD,LPCWSTR,DWORD);
+BOOL WINAPI FtpCommandA(HINTERNET,BOOL,DWORD,LPCSTR,DWORD_PTR,HINTERNET*);
+BOOL WINAPI FtpCommandW(HINTERNET,BOOL,DWORD,LPCWSTR,DWORD_PTR,HINTERNET*);
 BOOL WINAPI GopherCreateLocatorA(LPCSTR,INTERNET_PORT,LPCSTR,LPCSTR,DWORD,LPSTR,PDWORD);
 BOOL WINAPI GopherCreateLocatorW(LPCWSTR,INTERNET_PORT,LPCWSTR,LPCWSTR,DWORD,LPWSTR,PDWORD);
 BOOL WINAPI GopherGetLocatorTypeA(LPCSTR,PDWORD);
@@ -807,11 +857,22 @@ BOOL WINAPI InternetAutodial(DWORD,DWORD);
 BOOL WINAPI InternetAutodialHangup(DWORD);
 BOOL WINAPI InternetGetConnectedState(LPDWORD,DWORD);
 BOOL WINAPI InternetSetDialState(LPCTSTR,DWORD,DWORD);
+BOOL WINAPI InternetReadFileExA(HINTERNET,LPINTERNET_BUFFERSA,DWORD,DWORD_PTR);
+BOOL WINAPI InternetReadFileExW(HINTERNET,LPINTERNET_BUFFERSW,DWORD,DWORD_PTR);
+GROUPID WINAPI CreateUrlCacheGroup(DWORD,LPVOID);
+BOOL WINAPI DeleteUrlCacheGroup(GROUPID,DWORD,LPVOID);
+HANDLE WINAPI FindFirstUrlCacheGroup(DWORD,DWORD,LPVOID,DWORD,GROUPID*,LPVOID);
+BOOL WINAPI FindNextUrlCacheGroup(HANDLE,GROUPID*,LPVOID);
+BOOL WINAPI GetUrlCacheGroupAttributeA(GROUPID,DWORD,DWORD,LPINTERNET_CACHE_GROUP_INFOA,LPDWORD,LPVOID);
+BOOL WINAPI GetUrlCacheGroupAttributeW(GROUPID,DWORD,DWORD,LPINTERNET_CACHE_GROUP_INFOW,LPDWORD,LPVOID);
+BOOL WINAPI SetUrlCacheGroupAttributeA(GROUPID,DWORD,DWORD,LPINTERNET_CACHE_GROUP_INFOA,LPVOID);
+BOOL WINAPI SetUrlCacheGroupAttributeW(GROUPID,DWORD,DWORD,LPINTERNET_CACHE_GROUP_INFOW,LPVOID);
 
 #ifdef UNICODE
 #define InternetCrackUrl InternetCrackUrlW
 #define InternetCreateUrl InternetCreateUrlW
 #define InternetCanonicalizeUrl InternetCanonicalizeUrlW
+#define InternetCheckConnection InternetCheckConnectionW
 #define InternetCombineUrl InternetCombineUrlW
 #define InternetOpen InternetOpenW
 #define InternetConnect InternetConnectW
@@ -821,6 +882,7 @@ BOOL WINAPI InternetSetDialState(LPCTSTR,DWORD,DWORD);
 #define InternetSetOption InternetSetOptionW
 #define InternetSetOptionEx InternetSetOptionExW
 #define InternetGetLastResponseInfo InternetGetLastResponseInfoW
+#define InternetReadFileEx  InternetReadFileExW
 #define FtpFindFirstFile FtpFindFirstFileW
 #define FtpGetFile FtpGetFileW
 #define FtpPutFile FtpPutFileW
@@ -853,11 +915,14 @@ BOOL WINAPI InternetSetDialState(LPCTSTR,DWORD,DWORD);
 #define RetrieveUrlCacheEntryFile RetrieveUrlCacheEntryFileW
 #define HttpSendRequestEx	 HttpSendRequestExW
 #define HttpEndRequest		 HttpEndRequestW
+#define GetUrlCacheGroupAttribute  GetUrlCacheGroupAttributeW
+#define SetUrlCacheGroupAttribute  SetUrlCacheGroupAttributeW
 #else
 #define GopherGetAttribute GopherGetAttributeA
 #define InternetCrackUrl InternetCrackUrlA
 #define InternetCreateUrl InternetCreateUrlA
 #define InternetCanonicalizeUrl InternetCanonicalizeUrlA
+#define InternetCheckConnection InternetCheckConnectionA
 #define InternetCombineUrl InternetCombineUrlA
 #define InternetOpen InternetOpenA
 #define InternetConnect InternetConnectA
@@ -867,6 +932,7 @@ BOOL WINAPI InternetSetDialState(LPCTSTR,DWORD,DWORD);
 #define InternetSetOption InternetSetOptionA
 #define InternetSetOptionEx InternetSetOptionExA
 #define InternetGetLastResponseInfo InternetGetLastResponseInfoA
+#define InternetReadFileEx  InternetReadFileExA
 #define FtpFindFirstFile FtpFindFirstFileA
 #define FtpGetFile FtpGetFileA
 #define FtpPutFile FtpPutFileA
@@ -898,6 +964,8 @@ BOOL WINAPI InternetSetDialState(LPCTSTR,DWORD,DWORD);
 #define RetrieveUrlCacheEntryFile RetrieveUrlCacheEntryFileA
 #define HttpSendRequestEx	 HttpSendRequestExA
 #define HttpEndRequest		 HttpEndRequestA
+#define GetUrlCacheGroupAttribute  GetUrlCacheGroupAttributeA
+#define SetUrlCacheGroupAttribute  SetUrlCacheGroupAttributeA
 #endif /* UNICODE */
 #endif /* RC_INVOKED */
 #ifdef __cplusplus
