@@ -2,8 +2,8 @@
 
   Definitions for winsock 2
 
-  Initially taken from the Wine project.
-  
+  Contributed by the WINE project.
+
   Portions Copyright (c) 1980, 1983, 1988, 1993
   The Regents of the University of California.  All rights reserved.
 
@@ -13,6 +13,10 @@
 #if !(defined _WINSOCK2_H || defined _WINSOCK_H)
 #define _WINSOCK2_H
 #define _WINSOCK_H /* to prevent later inclusion of winsock.h */
+#if __GNUC__ >=3
+#pragma GCC system_header
+#endif
+
 #define _GNU_H_WINDOWS32_SOCKETS
 
 #include <windows.h>
@@ -41,7 +45,7 @@ typedef u_int	SOCKET;
 
 #ifndef _SYS_TYPES_FD_SET
 /* fd_set may be defined by the newlib <sys/types.h>
- * if __USE_W32_SOCKETS not defined.   
+ * if __USE_W32_SOCKETS not defined.
  */
 #ifdef fd_set
 #undef fd_set
@@ -89,13 +93,22 @@ if (__i == ((fd_set *)(set))->fd_count) {\
 #endif
 #elif !defined (USE_SYS_TYPES_FD_SET)
 #warning "fd_set and associated macros have been defined in sys/types.  \
-    This may cause runtime problems with W32 sockets" 
+    This may cause runtime problems with W32 sockets"
 #endif /* ndef _SYS_TYPES_FD_SET */
 #if !(defined (__INSIDE_CYGWIN__) || (__INSIDE_MSYS__))
+#ifndef _TIMEVAL_DEFINED /* also in sys/time.h */
+#define _TIMEVAL_DEFINED
 struct timeval {
 	long    tv_sec;
 	long    tv_usec;
 };
+#define timerisset(tvp)	 ((tvp)->tv_sec || (tvp)->tv_usec)
+#define timercmp(tvp, uvp, cmp) \
+	(((tvp)->tv_sec != (uvp)->tv_sec) ? \
+	((tvp)->tv_sec cmp (uvp)->tv_sec) : \
+	((tvp)->tv_usec cmp (uvp)->tv_usec))
+#define timerclear(tvp)	 (tvp)->tv_sec = (tvp)->tv_usec = 0
+#endif /* _TIMEVAL_DEFINED */
 struct  hostent {
 	char	*h_name;
 	char	**h_aliases;
@@ -108,12 +121,6 @@ struct linger {
 	u_short l_onoff;
 	u_short l_linger;
 };
-#define timerisset(tvp)	 ((tvp)->tv_sec || (tvp)->tv_usec)
-#define timercmp(tvp, uvp, cmp) \
-	(((tvp)->tv_sec != (uvp)->tv_sec) ? \
-	((tvp)->tv_sec cmp (uvp)->tv_sec) : \
-	((tvp)->tv_usec cmp (uvp)->tv_usec))
-#define timerclear(tvp)	 (tvp)->tv_sec = (tvp)->tv_usec = 0
 #endif /* ! (__INSIDE_CYGWIN__ || __INSIDE_MSYS__) */
 #define IOCPARM_MASK	0x7f
 #define IOC_VOID	0x20000000
@@ -167,6 +174,16 @@ struct  protoent {
 #define IPPROTO_ND	77
 #define IPPROTO_RAW	255
 #define IPPROTO_MAX	256
+/* IPv6 options */
+#define IPPROTO_HOPOPTS		0  /* IPv6 Hop-by-Hop options */
+#define IPPROTO_IPV6		41 /* IPv6 header */
+#define IPPROTO_ROUTING		43 /* IPv6 Routing header */
+#define IPPROTO_FRAGMENT	44 /* IPv6 fragmentation header */
+#define IPPROTO_ESP		50 /* encapsulating security payload */
+#define IPPROTO_AH		51 /* authentication header */
+#define IPPROTO_ICMPV6		58 /* ICMPv6 */
+#define IPPROTO_NONE		59 /* IPv6 no next header */
+#define IPPROTO_DSTOPTS		60 /* IPv6 Destination options */
 #define IPPORT_ECHO	7
 #define IPPORT_DISCARD	9
 #define IPPORT_SYSTAT	11
@@ -208,7 +225,7 @@ struct in_addr {
 #define s_impno S_un.S_un_b.s_b4
 #define s_lh    S_un.S_un_b.s_b3
 };
-#define IN_CLASSA(i)	((long)(i)&0x80000000)
+#define IN_CLASSA(i)	(((long)(i)&0x80000000) == 0)
 #define IN_CLASSA_NET	0xff000000
 #define IN_CLASSA_NSHIFT	24
 #define IN_CLASSA_HOST	0x00ffffff
@@ -257,6 +274,7 @@ typedef WSADATA *LPWSADATA;
 #define SO_LINGER	128
 #define SO_OOBINLINE	256
 #define SO_DONTLINGER	(u_int)(~SO_LINGER)
+#define SO_EXCLUSIVEADDRUSE ((u_int)(~SO_REUSEADDR))
 #define SO_SNDBUF	0x1001
 #define SO_RCVBUF	0x1002
 #define SO_SNDLOWAT	0x1003
@@ -304,13 +322,35 @@ typedef WSADATA *LPWSADATA;
 #define AF_CLUSTER  24
 #define AF_12844    25
 #define AF_IRDA     26
-#define AF_NETDES   28                     
+#define AF_NETDES   28
 #if !(defined (__INSIDE_CYGWIN__) || defined (__INSIDE_MSYS__))
 #define AF_MAX	29
 struct sockaddr {
 	u_short sa_family;
 	char	sa_data[14];
 };
+
+/* Portable IPv6/IPv4 version of sockaddr.  Based on RFC 2553.
+   Pad to force 8 byte alignment and maximum size of 128 bytes. */
+
+/*
+ * Desired design of maximum size and alignment
+ */
+#define _SS_MAXSIZE    128
+#define _SS_ALIGNSIZE  (sizeof (__int64)) 
+/*
+ * Definitions used for sockaddr_storage structure paddings design.
+ */
+#define _SS_PAD1SIZE   (_SS_ALIGNSIZE - sizeof (short))
+#define _SS_PAD2SIZE   (_SS_MAXSIZE - (sizeof (short) \
+				       + _SS_PAD1SIZE \
+				       + _SS_ALIGNSIZE))
+struct sockaddr_storage {
+    short ss_family;
+    char __ss_pad1[_SS_PAD1SIZE];  /* pad to 8 */
+    __int64 __ss_align;  	   /* force alignment */
+    char __ss_pad2[_SS_PAD2SIZE];  /*  pad to 128 */
+};
 #endif /* ! (__INSIDE_CYGWIN__ || __INSIDE_MSYS__) */
 
 struct sockproto {
@@ -355,7 +395,7 @@ struct sockproto {
 #define MAXGETHOSTSTRUCT	1024
 
 #define FD_READ_BIT      0
-#define FD_READ          (1 << FD_READ_BIT)  
+#define FD_READ          (1 << FD_READ_BIT)
 #define FD_WRITE_BIT     1
 #define FD_WRITE         (1 << FD_WRITE_BIT)
 #define FD_OOB_BIT       2
@@ -378,6 +418,7 @@ struct sockproto {
 #define FD_MAX_EVENTS    10
 #define FD_ALL_EVENTS    ((1 << FD_MAX_EVENTS) - 1)
 
+#ifndef WSABASEERR
 #define WSABASEERR	10000
 #define WSAEINTR	(WSABASEERR+4)
 #define WSAEBADF	(WSABASEERR+9)
@@ -386,7 +427,7 @@ struct sockproto {
 #define WSAEINVAL	(WSABASEERR+22)
 #define WSAEMFILE	(WSABASEERR+24)
 #define WSAEWOULDBLOCK	(WSABASEERR+35)
-#define WSAEINPROGRESS	(WSABASEERR+36) /* deprecated on WinSock2 */ 
+#define WSAEINPROGRESS	(WSABASEERR+36) /* deprecated on WinSock2 */
 #define WSAEALREADY	(WSABASEERR+37)
 #define WSAENOTSOCK	(WSABASEERR+38)
 #define WSAEDESTADDRREQ	(WSABASEERR+39)
@@ -430,6 +471,51 @@ struct sockproto {
 #define WSATRY_AGAIN	(WSABASEERR+1002)
 #define WSANO_RECOVERY	(WSABASEERR+1003)
 #define WSANO_DATA	(WSABASEERR+1004)
+
+/* WinSock2 specific error codes */
+#define WSAENOMORE	(WSABASEERR+102)
+#define WSAECANCELLED	(WSABASEERR+103)
+#define WSAEINVALIDPROCTABLE	(WSABASEERR+104)
+#define WSAEINVALIDPROVIDER	(WSABASEERR+105)
+#define WSAEPROVIDERFAILEDINIT	(WSABASEERR+106)
+#define WSASYSCALLFAILURE	(WSABASEERR+107)
+#define WSASERVICE_NOT_FOUND	(WSABASEERR+108)
+#define WSATYPE_NOT_FOUND	(WSABASEERR+109)
+#define WSA_E_NO_MORE	(WSABASEERR+110)
+#define WSA_E_CANCELLED	(WSABASEERR+111)
+#define WSAEREFUSED	(WSABASEERR+112)
+
+/* WS QualityofService errors */
+#define WSA_QOS_RECEIVERS	(WSABASEERR + 1005)
+#define WSA_QOS_SENDERS	(WSABASEERR + 1006)
+#define WSA_QOS_NO_SENDERS	(WSABASEERR + 1007)
+#define WSA_QOS_NO_RECEIVERS	(WSABASEERR + 1008)
+#define WSA_QOS_REQUEST_CONFIRMED	(WSABASEERR + 1009)
+#define WSA_QOS_ADMISSION_FAILURE	(WSABASEERR + 1010)
+#define WSA_QOS_POLICY_FAILURE	(WSABASEERR + 1011)
+#define WSA_QOS_BAD_STYLE	(WSABASEERR + 1012)
+#define WSA_QOS_BAD_OBJECT	(WSABASEERR + 1013)
+#define WSA_QOS_TRAFFIC_CTRL_ERROR	(WSABASEERR + 1014)
+#define WSA_QOS_GENERIC_ERROR	(WSABASEERR + 1015)
+#define WSA_QOS_ESERVICETYPE	(WSABASEERR + 1016)
+#define WSA_QOS_EFLOWSPEC	(WSABASEERR + 1017)
+#define WSA_QOS_EPROVSPECBUF	(WSABASEERR + 1018)
+#define WSA_QOS_EFILTERSTYLE	(WSABASEERR + 1019)
+#define WSA_QOS_EFILTERTYPE	(WSABASEERR + 1020)
+#define WSA_QOS_EFILTERCOUNT	(WSABASEERR + 1021)
+#define WSA_QOS_EOBJLENGTH	(WSABASEERR + 1022)
+#define WSA_QOS_EFLOWCOUNT	(WSABASEERR + 1023)
+#define WSA_QOS_EUNKOWNPSOBJ	(WSABASEERR + 1024)
+#define WSA_QOS_EPOLICYOBJ	(WSABASEERR + 1025)
+#define WSA_QOS_EFLOWDESC	(WSABASEERR + 1026)
+#define WSA_QOS_EPSFLOWSPEC	(WSABASEERR + 1027)
+#define WSA_QOS_EPSFILTERSPEC	(WSABASEERR + 1028)
+#define WSA_QOS_ESDMODEOBJ	(WSABASEERR + 1029)
+#define WSA_QOS_ESHAPERATEOBJ	(WSABASEERR + 1030)
+#define WSA_QOS_RESERVED_PETYPE	(WSABASEERR + 1031)
+
+#endif /* !WSABASEERR */
+
 #define WSANO_ADDRESS	WSANO_DATA
 #if !(defined (__INSIDE_CYGWIN__) || defined (__INSIDE_MSYS__))
 #define h_errno WSAGetLastError()
@@ -463,19 +549,58 @@ DECLARE_STDCALL_P(struct servent *) getservbyport(int,const char*);
 DECLARE_STDCALL_P(struct servent *) getservbyname(const char*,const char*);
 DECLARE_STDCALL_P(struct protoent *) getprotobynumber(int);
 DECLARE_STDCALL_P(struct protoent *) getprotobyname(const char*);
+typedef SOCKET (PASCAL * LPFN_ACCEPT)(SOCKET, struct sockaddr*,int *);
+typedef int (PASCAL * LPFN_BIND)(SOCKET, const struct sockaddr*,int);
+typedef int (PASCAL * LPFN_CLOSESOCKET)(SOCKET);
+typedef int (PASCAL * LPFN_CONNECT)(SOCKET, const struct sockaddr*,int);
+typedef int (PASCAL * LPFN_IOCTLSOCKET)(SOCKET, long, u_long*);
+typedef int	(PASCAL * LPFN_GETPEERNAME)(SOCKET, struct sockaddr*, int*);
+typedef int(PASCAL * LPFN_GETSOCKNAME)(SOCKET, struct sockaddr*, int*);
+typedef int(PASCAL * LPFN_GETSOCKOPT)(SOCKET, int, int, char*, int*);
+typedef u_long(PASCAL * LPFN_HTONL)(u_long);
+typedef u_short(PASCAL * LPFN_HTONS)(u_short);
+typedef unsigned long(PASCAL * LPFN_INET_ADDR)(const char*);
+typedef char*(PASCAL * LPFN_INET_NTOA)(struct in_addr);
+typedef int(PASCAL * LPFN_LISTEN)(SOCKET, int);
+typedef u_long(PASCAL * LPFN_NTOHL)(u_long);
+typedef u_short(PASCAL * LPFN_NTOHS)(u_short);
+typedef int(PASCAL * LPFN_RECV)(SOCKET, char*, int, int);
+typedef int(PASCAL * LPFN_RECVFROM)(SOCKET, char*, int, int, struct sockaddr*,  int*);
+typedef int(PASCAL * LPFN_SELECT)(int, fd_set*, fd_set*, fd_set*, const struct timeval*);
+typedef int(PASCAL * LPFN_SEND)(SOCKET, const char*, int, int);
+typedef int(PASCAL * LPFN_SENDTO)(SOCKET, const char*, int, int, const struct sockaddr*, int);
+typedef int(PASCAL * LPFN_SETSOCKOPT)(SOCKET, int, int, const char*, int);
+typedef int(PASCAL * LPFN_SHUTDOWN)(SOCKET, int);
+typedef SOCKET(PASCAL * LPFN_SOCKET)(int, int, int);
+typedef struct hostent*(PASCAL * LPFN_GETHOSTBYADDR)( const char*, int, int);
+typedef struct hostent*(PASCAL * LPFN_GETHOSTBYNAME)( const char*);
+typedef int(PASCAL * LPFN_GETHOSTNAME)(char*, int);
+typedef struct servent*(PASCAL * LPFN_GETSERVBYPORT)(int, const char*);
+typedef struct servent*(PASCAL * LPFN_GETSERVBYNAME)(const char*, const char*);
+typedef struct protoent*(PASCAL * LPFN_GETPROTOBYNUMBER)(int);
+typedef struct protoent*(PASCAL * LPFN_GETPROTOBYNAME)(const char*);
+
 int PASCAL WSAStartup(WORD,LPWSADATA);
 int PASCAL WSACleanup(void);
 void PASCAL WSASetLastError(int);
 int PASCAL WSAGetLastError(void);
+typedef int(PASCAL * LPFN_WSASTARTUP)(WORD, LPWSADATA);
+typedef int(PASCAL * LPFN_WSACLEANUP)(void);
+typedef void(PASCAL * LPFN_WSASETLASTERROR)(int);
+typedef int(PASCAL * LPFN_WSAGETLASTERROR)(void);
 /*
- * Pesudo-blocking functions N/A in ws2_32.dll (use threads instead)
+ * Pseudo-blocking functions are deprecated in WinSock2
+ * spec. Use threads instead.
  */
-#if 0
 BOOL PASCAL WSAIsBlocking(void);
 int PASCAL WSAUnhookBlockingHook(void);
 FARPROC PASCAL WSASetBlockingHook(FARPROC);
 int PASCAL WSACancelBlockingCall(void);
-#endif
+typedef BOOL(PASCAL * LPFN_WSAISBLOCKING)(void);
+typedef int(PASCAL * LPFN_WSAUNHOOKBLOCKINGHOOK)(void);
+typedef FARPROC (PASCAL * LPFN_WSASETBLOCKINGHOOK)(FARPROC);
+typedef int(PASCAL * LPFN_WSACANCELBLOCKINGCALL)(void);
+
 HANDLE PASCAL WSAAsyncGetServByName(HWND,u_int,const char*,const char*,char*,int);
 HANDLE PASCAL WSAAsyncGetServByPort(HWND,u_int,int,const char*,char*,int);
 HANDLE PASCAL WSAAsyncGetProtoByName(HWND,u_int,const char*,char*,int);
@@ -484,15 +609,23 @@ HANDLE PASCAL WSAAsyncGetHostByName(HWND,u_int,const char*,char*,int);
 HANDLE PASCAL WSAAsyncGetHostByAddr(HWND,u_int,const char*,int,int,char*,int);
 int PASCAL WSACancelAsyncRequest(HANDLE);
 int PASCAL WSAAsyncSelect(SOCKET,HWND,u_int,long);
+typedef HANDLE(PASCAL * LPFN_WSAASYNCGETSERVBYNAME)(HWND, u_int, const char *, const char *, char *, int);
+typedef HANDLE(PASCAL * LPFN_WSAASYNCGETSERVBYPORT)(HWND, u_int, int, const char *, char *, int);
+typedef HANDLE(PASCAL * LPFN_WSAASYNCGETPROTOBYNAME)(HWND, u_int, const char*, char*, int);
+typedef HANDLE(PASCAL * LPFN_WSAASYNCGETPROTOBYNUMBER)(HWND, u_int, int, char*, int);
+typedef HANDLE(PASCAL * LPFN_WSAASYNCGETHOSTBYADDR)(HWND, u_int, const char*, int, int, char*, int);
+typedef int(PASCAL * LPFN_WSACANCELASYNCREQUEST)(HANDLE);
+typedef int(PASCAL * LPFN_WSAASYNCSELECT)(SOCKET, HWND, u_int, long);
 #if ! (defined (__INSIDE_CYGWIN__) || defined (__INSIDE_MSYS__))
 u_long PASCAL htonl(u_long);
 u_long PASCAL ntohl(u_long);
 u_short PASCAL htons(u_short);
 u_short PASCAL ntohs(u_short);
 int PASCAL select(int nfds,fd_set*,fd_set*,fd_set*,const struct timeval*);
-int PASCAL gethostname(char*,int);
 #endif /* ! (__INSIDE_CYGWIN__ || __INSIDE_MSYS__) */
 
+int PASCAL gethostname(char*,int);
+
 #define WSAMAKEASYNCREPLY(b,e)	MAKELONG(b,e)
 #define WSAMAKESELECTREPLY(e,error)	MAKELONG(e,error)
 #define WSAGETASYNCBUFLEN(l)	LOWORD(l)
@@ -503,6 +636,7 @@ int PASCAL gethostname(char*,int);
 typedef struct sockaddr SOCKADDR;
 typedef struct sockaddr *PSOCKADDR;
 typedef struct sockaddr *LPSOCKADDR;
+typedef struct sockaddr_storage SOCKADDR_STORAGE, *PSOCKADDR_STORAGE;
 typedef struct sockaddr_in SOCKADDR_IN;
 typedef struct sockaddr_in *PSOCKADDR_IN;
 typedef struct sockaddr_in *LPSOCKADDR_IN;
@@ -555,50 +689,6 @@ typedef struct timeval *LPTIMEVAL;
 #define	MSG_INTERRUPT	0x10
 #define	MSG_MAXIOVLEN	16
 
-/* WinSock2 specific error codes */
-#define	WSAEDISCON	(WSABASEERR+101)
-#define	WSAENOMORE	(WSABASEERR+102)
-#define	WSAECANCELLED	(WSABASEERR+103)
-#define	WSAEINVALIDPROCTABLE	(WSABASEERR+104)
-#define	WSAEINVALIDPROVIDER	(WSABASEERR+105)
-#define	WSAEPROVIDERFAILEDINIT	(WSABASEERR+106)
-#define	WSASYSCALLFAILURE	(WSABASEERR+107)
-#define	WSASERVICE_NOT_FOUND	(WSABASEERR+108)
-#define	WSATYPE_NOT_FOUND	(WSABASEERR+109)
-#define	WSA_E_NO_MORE	(WSABASEERR+110)
-#define	WSA_E_CANCELLED	(WSABASEERR+111)
-#define	WSAEREFUSED	(WSABASEERR+112)
- 
-/* WS QualityofService errors */
-#define  WSA_QOS_RECEIVERS  (WSABASEERR + 1005)
-#define  WSA_QOS_SENDERS    (WSABASEERR + 1006)
-#define  WSA_QOS_NO_SENDERS (WSABASEERR + 1007)
-#define  WSA_QOS_NO_RECEIVERS   (WSABASEERR + 1008)
-#define  WSA_QOS_REQUEST_CONFIRMED  (WSABASEERR + 1009)
-#define  WSA_QOS_ADMISSION_FAILURE  (WSABASEERR + 1010)
-#define  WSA_QOS_POLICY_FAILURE (WSABASEERR + 1011)
-#define  WSA_QOS_BAD_STYLE  (WSABASEERR + 1012)
-#define  WSA_QOS_BAD_OBJECT (WSABASEERR + 1013)
-#define  WSA_QOS_TRAFFIC_CTRL_ERROR (WSABASEERR + 1014)
-#define  WSA_QOS_GENERIC_ERROR  (WSABASEERR + 1015)
-#define  WSA_QOS_ESERVICETYPE   (WSABASEERR + 1016)
-#define  WSA_QOS_EFLOWSPEC	(WSABASEERR + 1017)
-#define  WSA_QOS_EPROVSPECBUF    (WSABASEERR + 1018)
-#define  WSA_QOS_EFILTERSTYLE   (WSABASEERR + 1019)
-#define  WSA_QOS_EFILTERTYPE    (WSABASEERR + 1020)
-#define  WSA_QOS_EFILTERCOUNT   (WSABASEERR + 1021)
-#define  WSA_QOS_EOBJLENGTH (WSABASEERR + 1022)
-#define  WSA_QOS_EFLOWCOUNT (WSABASEERR + 1023)
-#define  WSA_QOS_EUNKOWNPSOBJ   (WSABASEERR + 1024)
-#define  WSA_QOS_EPOLICYOBJ (WSABASEERR + 1025)
-#define  WSA_QOS_EFLOWDESC  (WSABASEERR + 1026)
-#define  WSA_QOS_EPSFLOWSPEC    (WSABASEERR + 1027)
-#define  WSA_QOS_EPSFILTERSPEC  (WSABASEERR + 1028)
-#define  WSA_QOS_ESDMODEOBJ (WSABASEERR + 1029)
-#define  WSA_QOS_ESHAPERATEOBJ  (WSABASEERR + 1030)
-#define  WSA_QOS_RESERVED_PETYPE    (WSABASEERR + 1031)
-
- 
 #define	WSAAPI	WINAPI
 #define WSAEVENT	HANDLE
 #define	LPWSAEVENT	LPHANDLE
@@ -634,19 +724,28 @@ typedef enum
 	GuaranteedService
 } GUARANTEE;
 
-typedef long int32;
+/* TODO: FLOWSPEC and related definitions belong in qos.h */
 
+/*
+   Windows Sockets 2 Application Programming Interface,
+   revision 2.2.2 (1997) uses the type uint32 for SERVICETYPE
+   and the elements of _flowspec, but the type uint32 is not defined
+   or used anywhere else in the w32api. For now, just use
+   unsigned int, which is 32 bits on _WIN32 and _WIN64.
+*/
+
+typedef unsigned int	SERVICETYPE;
 typedef struct _flowspec
 {
-	int32	TokenRate;
-	int32	TokenBucketSize;
-	int32	PeakBandwidth;
-	int32	Latency;
-	int32	DelayVariation;
-	GUARANTEE	LevelOfGuarantee;
-	int32	CostOfCall;
-	int32	NetworkAvailability;
-} FLOWSPEC, *LPFLOWSPEC;
+	unsigned int	TokenRate;
+	unsigned int	TokenBucketSize;
+	unsigned int	PeakBandwidth;
+	unsigned int	Latency;
+	unsigned int	DelayVariation;
+	SERVICETYPE	ServiceType;
+	unsigned int	MaxSduSize;
+	unsigned int	MinimumPolicedSize;
+   } FLOWSPEC, *PFLOWSPEC, *LPFLOWSPEC;
 
 typedef struct _QualityOfService
 {
@@ -687,7 +786,6 @@ typedef struct _AFPROTOCOLS {
 	INT iProtocol;
 } AFPROTOCOLS, *PAFPROTOCOLS, *LPAFPROTOCOLS;
 
-
 typedef enum _WSAEcomparator
 {
 	COMP_EQUAL = 0,
@@ -700,12 +798,26 @@ typedef struct _WSAVersion
 	WSAECOMPARATOR	ecHow;
 } WSAVERSION, *PWSAVERSION, *LPWSAVERSION;
 
-/*
- * nspapi.h has definition of  LPCSADDR_INFO, needed in WSAQuery
- * but itself needs LPSOCKADDR which is defined earlier in this file 
-*/
-#include <nspapi.h>
-#ifndef __BLOB_T_DEFINED /* also in wtypes.h */
+#ifndef __CSADDR_T_DEFINED /* also in nspapi.h */
+#define __CSADDR_T_DEFINED
+typedef struct _SOCKET_ADDRESS {
+	LPSOCKADDR lpSockaddr;
+	INT iSockaddrLength;
+} SOCKET_ADDRESS,*PSOCKET_ADDRESS,*LPSOCKET_ADDRESS;
+typedef struct _CSADDR_INFO {
+	SOCKET_ADDRESS LocalAddr;
+	SOCKET_ADDRESS RemoteAddr;
+	INT iSocketType;
+	INT iProtocol;
+} CSADDR_INFO,*PCSADDR_INFO,*LPCSADDR_INFO;
+#endif
+
+typedef struct _SOCKET_ADDRESS_LIST {
+    INT             iAddressCount;
+    SOCKET_ADDRESS  Address[1];
+} SOCKET_ADDRESS_LIST, * LPSOCKET_ADDRESS_LIST;
+
+#ifndef __BLOB_T_DEFINED /* also in wtypes.h and nspapi.h */
 #define __BLOB_T_DEFINED
 typedef struct _BLOB {
 	ULONG	cbSize;
@@ -761,6 +873,23 @@ typedef PWSAQUERYSETA PWSAQUERYSET;
 typedef LPWSAQUERYSETA LPWSAQUERYSET;
 #endif
 
+#define LUP_DEEP                0x0001
+#define LUP_CONTAINERS          0x0002
+#define LUP_NOCONTAINERS        0x0004
+#define LUP_NEAREST             0x0008
+#define LUP_RETURN_NAME         0x0010
+#define LUP_RETURN_TYPE         0x0020
+#define LUP_RETURN_VERSION      0x0040
+#define LUP_RETURN_COMMENT      0x0080
+#define LUP_RETURN_ADDR         0x0100
+#define LUP_RETURN_BLOB         0x0200
+#define LUP_RETURN_ALIASES      0x0400
+#define LUP_RETURN_QUERY_STRING 0x0800
+#define LUP_RETURN_ALL          0x0FF0
+#define LUP_RES_SERVICE         0x8000
+#define LUP_FLUSHCACHE          0x1000
+#define LUP_FLUSHPREVIOUS       0x2000
+
 typedef struct _WSANSClassInfoA
 {
 	LPSTR	lpszName;
@@ -906,6 +1035,37 @@ typedef WSAPROTOCOL_INFOA WSAPROTOCOL_INFO;
 typedef LPWSAPROTOCOL_INFOA LPWSAPROTOCOL_INFO;
 #endif
 
+/* Needed for XP & .NET Server function WSANSPIoctl.  */
+typedef enum _WSACOMPLETIONTYPE {
+    NSP_NOTIFY_IMMEDIATELY = 0,
+    NSP_NOTIFY_HWND,
+    NSP_NOTIFY_EVENT,
+    NSP_NOTIFY_PORT,
+    NSP_NOTIFY_APC
+} WSACOMPLETIONTYPE, * PWSACOMPLETIONTYPE, * LPWSACOMPLETIONTYPE;
+typedef struct _WSACOMPLETION {
+    WSACOMPLETIONTYPE Type;
+    union {
+        struct {
+            HWND hWnd;
+            UINT uMsg;
+            WPARAM context;
+        } WindowMessage;
+        struct {
+            LPWSAOVERLAPPED lpOverlapped;
+        } Event;
+        struct {
+            LPWSAOVERLAPPED lpOverlapped;
+            LPWSAOVERLAPPED_COMPLETION_ROUTINE lpfnCompletionProc;
+        } Apc;
+        struct {
+            LPWSAOVERLAPPED lpOverlapped;
+            HANDLE hPort;
+            ULONG_PTR Key;
+        } Port;
+    } Parameters;
+} WSACOMPLETION, *PWSACOMPLETION, *LPWSACOMPLETION;
+
 #define PFL_MULTIPLE_PROTO_ENTRIES          0x00000001
 #define PFL_RECOMMENDED_PROTO_ENTRY         0x00000002
 #define PFL_HIDDEN                          0x00000004
@@ -970,6 +1130,8 @@ typedef LPWSAPROTOCOL_INFOA LPWSAPROTOCOL_INFO;
 #define SIO_ADDRESS_LIST_QUERY        _WSAIOR(IOC_WS2,22)
 #define SIO_ADDRESS_LIST_CHANGE       _WSAIO(IOC_WS2,23)
 #define SIO_QUERY_TARGET_PNP_HANDLE   _WSAIOR(IOC_WS2,24)
+#define SIO_NSP_NOTIFY_CHANGE         _WSAIOW(IOC_WS2,25)
+
 #define TH_NETDEV	0x00000001
 #define TH_TAPI	0x00000002
 
@@ -1004,6 +1166,7 @@ INT WINAPI WSALookupServiceBeginW(LPWSAQUERYSETW lpqsRestrictions, DWORD, LPHAND
 INT WINAPI WSALookupServiceNextA(HANDLE, DWORD, LPDWORD, LPWSAQUERYSETA);
 INT WINAPI WSALookupServiceNextW(HANDLE, DWORD, LPDWORD, LPWSAQUERYSETW);
 INT WINAPI WSALookupServiceEnd(HANDLE);
+int WINAPI WSANSPIoctl(HANDLE,DWORD,LPVOID,DWORD,LPVOID,DWORD,LPDWORD,LPWSACOMPLETION); /* XP or .NET Server */
 int WINAPI WSANtohl(SOCKET, unsigned long, unsigned long *);
 int WINAPI WSANtohs(SOCKET, unsigned short, unsigned short *);
 int WINAPI WSARecv(SOCKET, LPWSABUF, DWORD, LPDWORD, LPDWORD, LPWSAOVERLAPPED, LPWSAOVERLAPPED_COMPLETION_ROUTINE);
@@ -1022,7 +1185,6 @@ SOCKET WINAPI WSASocketW(int, int, int, LPWSAPROTOCOL_INFOW, GROUP, DWORD);
 INT WINAPI WSAStringToAddressA(LPSTR, INT, LPWSAPROTOCOL_INFOA, LPSOCKADDR, LPINT);
 INT WINAPI WSAStringToAddressW(LPWSTR, INT, LPWSAPROTOCOL_INFOW, LPSOCKADDR, LPINT);
 DWORD WINAPI WSAWaitForMultipleEvents(DWORD, const WSAEVENT *, BOOL, DWORD, BOOL);
-
 typedef SOCKET (WINAPI *LPFN_WSAACCEPT)(SOCKET, struct sockaddr *, LPINT, LPCONDITIONPROC, DWORD);
 typedef INT (WINAPI *LPFN_WSAADDRESSTOSTRINGA)(LPSOCKADDR, DWORD, LPWSAPROTOCOL_INFOA, LPSTR, LPDWORD);
 typedef INT (WINAPI *LPFN_WSAADDRESSTOSTRINGW)(LPSOCKADDR, DWORD, LPWSAPROTOCOL_INFOW, LPWSTR, LPDWORD);
@@ -1054,6 +1216,7 @@ typedef INT (WINAPI *LPFN_WSALOOKUPSERVICEBEGINW)(LPWSAQUERYSETW, DWORD, LPHANDL
 typedef INT (WINAPI *LPFN_WSALOOKUPSERVICENEXTA)(HANDLE, DWORD, LPDWORD, LPWSAQUERYSETA);
 typedef INT (WINAPI *LPFN_WSALOOKUPSERVICENEXTW)(HANDLE, DWORD, LPDWORD, LPWSAQUERYSETW);
 typedef INT (WINAPI *LPFN_WSALOOKUPSERVICEEND)(HANDLE);
+typedef int (WINAPI *LPFN_WSANSPIoctl)(HANDLE, DWORD,LPVOID,DWORD,LPVOID,DWORD,LPDWORD,LPWSACOMPLETION);
 typedef int (WINAPI *LPFN_WSANTOHL)(SOCKET, unsigned long, unsigned long *);
 typedef int (WINAPI *LPFN_WSANTOHS)(SOCKET, unsigned short, unsigned short *);
 typedef int (WINAPI *LPFN_WSARECV)(SOCKET, LPWSABUF, DWORD, LPDWORD, LPDWORD, LPWSAOVERLAPPED, LPWSAOVERLAPPED_COMPLETION_ROUTINE);
